package com.tuoheng.manage.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.api.R;
import com.sun.javaws.CacheUtil;
import com.tuoheng.common.core.common.BaseController;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.manage.dto.ResetPwdDto;
import com.tuoheng.manage.dto.UpdateUserInfoDto;
import com.tuoheng.manage.entity.User;
import com.tuoheng.manage.service.IUserService;
import com.tuoheng.manage.vo.UserVO;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import java.util.Map;

/**
 * 用户相关代码
 */
@RestController
@RequestMapping("/user-service")
public class UserController extends BaseController {

    @Autowired
    private IUserService userService;


    /**
     * 查询单条
     */
    @ApiOperation(value = "查看详情", notes = "传入id")
    @GetMapping("/user/{userId}")
    public R<UserVO> detail(@PathVariable("userId") String userId) {
        User detail = userService.getOne(Condition.getQueryWrapper(user));
        return JsonResult.success(UserWrapper.build().entityVO(detail));
    }

    /**
     * 查询单条
     */
    @ApiOperation(value = "查看详情", notes = "传入id")
    @GetMapping("/info")
    public R<UserVO> info(User user) {
        User detail = userService.getById(user.getUserId());
        return R.data(UserWrapper.build().entityVO(detail));
    }

    /**
     * 用户列表
     */
    @GetMapping("/list")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "account", value = "账号名", paramType = "query", dataType = "string"),
            @ApiImplicitParam(name = "realName", value = "姓名", paramType = "query", dataType = "string")
    })
    //@ApiOperationSupport(order = 3)
    @ApiOperation(value = "列表", notes = "传入account和realName")
    public R<IPage<UserVO>> list(@ApiIgnore @RequestParam Map<String, Object> user, Query query, BladeUser bladeUser) {
        QueryWrapper<User> queryWrapper = Condition.getQueryWrapper(user, User.class);
        IPage<User> pages = userService.page(Condition.getPage(query), (!bladeUser.getTenantId().equals(BladeConstant.ADMIN_TENANT_ID)) ? queryWrapper.lambda().eq(User::getTenantId, bladeUser.getTenantId()) : queryWrapper);
        return R.data(UserWrapper.build().pageVO(pages));
    }

    /**
     * 新增或修改
     */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入User")
    @PreAuth(RoleConstant.HAS_ROLE_ADMIN)
    public R submit(@Valid @RequestBody User user) {
        CacheUtil.clear(USER_CACHE);
        return R.status(userService.submit(user));
    }

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    @PutMapping("/edit")
    public JsonResult edit(@RequestBody UpdateUserInfoDto entity) {
        return userService.edit(entity);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PutMapping("/status")
    public JsonResult status(@RequestBody User entity) {
        return userService.setStatus(entity);
    }

    /**
     * 删除
     */
    @PostMapping("/remove")
    @ApiOperation(value = "删除", notes = "传入id集合")
    @DeleteMapping("/{userIds}")
    public JsonResult delete(@PathVariable("userIds") Integer[] userIds) {
        return userService.deleteByIds(userIds);
    }

    /**
     * 重置密码
     *
     * @param resetPwdDto 参数
     * @return
     */
    @PutMapping("/reset-password")
    @ApiOperation(value = "初始化密码", notes = "传入userId")
    @ApiImplicitParam(name = "id", value = "用户Id", paramType = "query", dataType = "string")
    public JsonResult resetPwd(@RequestBody ResetPwdDto resetPwdDto) {
        return userService.resetPwd(resetPwdDto);
    }



}
